#ifndef _CUBE_H

#define _CUBE_H

#define f32 float
#define u8	unsigned char
#define s16 signed short
#define u16 unsigned short
#define s32 signed long
#define u32 unsigned long

#define BPS 16

int get16bit(unsigned char* p);

int get32bit(unsigned char* p);

// both header and file type
typedef enum {
	type_std,
	type_sfass,
	type_mp2,
	type_pm2,
	type_halp,
	type_mp2d,
	type_idsp,
	type_spt,
	type_adx03,
	type_adx04,
	type_adp
} filetype;

// structure for a single channel
typedef struct {
	HANDLE infile; // processing on a single channel needs this,also this allows for seperate L/R files to be played at once
	
	// header data
	u32 num_samples;
	u32 num_adpcm_nibbles;
	u32 sample_rate;
	u16 loop_flag;
	u16 format;
	u32 sa,ea,ca;
	s16 coef[16];
	u16 gain; // never used anyway
	u16 ps,yn1,yn2;
	u16 lps,lyn1,lyn2;
	
	short chanbuf[0x8000/8*14];
	int readloc,writeloc; // offsets into the chanbuf
	filetype type;
	short bps; // bits per "size", 4 if offsets specified in nibbles, 8 if bytes
	u32 chanstart; // offset in file of start of this channel
	u32 offs; // current location
	short hist1,hist2; // sample history
	float fhist1,fhist2; // floating point sample history (for ADP decoder)
	long interleave; // _bytes_ of interleave, 0 if none
} CUBESTREAM;

// structure represents a DSP file
typedef struct {
	CUBESTREAM ch[2];
	int NCH;
	long nrsamples;
	long file_length;
	long nexthalp; // next HALPST header
	long halpsize;
	int lastchunk;
	long startinterleave;
} CUBEFILE;

int InitDSPFILE(char * infile, CUBEFILE * dsp);
int InitADXFILE(char * infile, CUBEFILE * adx);

void CloseCUBEFILE(CUBEFILE * dsp);

int DSPdecodebuffer
(
    u8			*input, // location of encoded source samples
    s16         *out,   // location of destination buffer (16 bits / sample)
    short		coef[16],   // location of decode coefficients
	short * histp,
	short * hist2p
);

int ADXdecodebuffer
(
    u8			*input, // location of encoded source samples
    s16         *out,   // location of destination buffer (16 bits / sample)
	short * histp,
	short * hist2p
);

int ADPdecodebuffer(
	unsigned char *input,
	short *outl,
	short *outr,
	float *histl1,
	float *histl2,
	float *histr1,
	float *histr2
);


void fillbuffers(CUBEFILE * dsp);

void fillbufferDSP(CUBESTREAM * stream);
void fillbufferDSPinterleave(CUBESTREAM * stream);
void fillbufferHALP(CUBESTREAM * stream);
void fillbufferADX(CUBEFILE * adx);

#endif